#
# Description:
# This scripts loads the statistics files of all clients.
# It calculates:
#   - The average CPU usage of the OS for each day
#   - The average CPU usage of all Jobs for each day
#   - The total number of frames rendered for each day
#
# And saves them as .csv file, which you can load into Excel.
#
# You need to execute this python script with rrPythonconsole
#
import os
import sys

mod_dir = os.path.dirname(__file__)
if mod_dir not in sys.path:
    sys.path.append(mod_dir)

from rr_python_utils.load_rrdata import rrData
from os import listdir
from os.path import isfile, join
from datetime import datetime
import time



exportFolder = rrGlobal.rrRootPath() + "inhouse/"
# Note: Please check your license via rrControl/help menu of you have access to the class rrTable
rrTableClient = rrGlobal._rrTable()
rrTableClient.setHeader(0, "Client")
rrTableClient.setHeader(1, "Was user ever logged out?")
rrTableClient.setHeader(2, "CPU usage percent")
rrTableClient.setHeader(3, "GPU usage percent")
rrTableClient.setHeader(4, "Total memory usage in MB, divided by 10.")
rrTableClient.setHeader(5, "Frames done")
rrTableClient.setHeader(6, "Job crashes")
rrTableClient.setHeader(7, "Job aborts")
rrTableClient.setHeader(8, "Job successes")


# _StatsRenderTime

clientStats = rrData._clientStats_data()
statFolder = rrGlobal.rrRootPath() + "sub/stats/clients/"
statFileList = [f for f in listdir(statFolder) if isfile(join(statFolder, f))]
print (statFolder + ": " + str(len(statFileList)) + " files found.")

loaded = 0
row = 1
num_days = 1

yesterday_DaysSinceEpoch = time.mktime(time.localtime()) /24/60/60
yesterday_DaysSinceEpoch = int( yesterday_DaysSinceEpoch -1 )
print("yesterday_DaysSinceEpoch:  " + str(yesterday_DaysSinceEpoch))

for statFile in statFileList:
    if (statFile.find(".stdb") < 0):
        continue
    if not clientStats.loadFromFile(statFolder + statFile):
        print(statFile + " | Error:  unable to load stats file! " + clientStats.getError())
    else:
        print(statFile)
        clientName = statFile[:-5]

        #clientStats.daysMax() -1 is the last entry in the array. If the client is running at the moment, it is the current day.
        #if the client was not running today, it might be yesterday. Which we want to export
        days = clientStats.daysMax() - 1
        sday = clientStats.days(days)
        log_date = datetime.utcfromtimestamp(sday.day *60*60*24)
        print("           " + str(log_date) + "    ("+ str(sday.day)+")")
        
        if (yesterday_DaysSinceEpoch != sday.day):
            days = clientStats.daysMax() - 2
            sday = clientStats.days(days)
            log_date = datetime.utcfromtimestamp(sday.day *60*60*24)
            print("           " + str(log_date) + "    ("+ str(sday.day)+")")
            if (yesterday_DaysSinceEpoch != sday.day):
                continue
        
        if (not sday.daySet):
            continue
            
        print("       EXPORT..." )
        rrTableClient.setString(0, row, clientName)
        cpu_amount_OS = 0
        gpu_total = 0
        system_memory_total = 0
        count = 0
        was_user_ever_logged_out = False
        for h in range(0, sday.dataMax()):
            sData = sday.data(h)
            if (not sData.hasData()):
                continue
            cpu_amount_OS = cpu_amount_OS + sData.cpuTotal
            gpu_total = gpu_total + sData.gpuTotal
            system_memory_total = system_memory_total + sData.memTotal10MB
            if not sData.userLoggedIn:
                was_user_ever_logged_out = True
            count = count + 1
        if (count > 0):
            cpu_amount_OS = cpu_amount_OS / count
            gpu_total = gpu_total / count
            system_memory_total = system_memory_total / count

        frame_count = 0
        job_crash_count = 0
        job_abort_count = 0
        job_success_count = 0
        for h in range(0, sday.dataHourlyMax()):
            hData = sday.dataHourly(h)
            frame_count += hData.jobThread(0).framesDone
            job_crash_count += hData.jobThread(0).jobCrashed
            job_abort_count += hData.jobThread(0).jobAborted
            job_success_count += hData.jobThread(0).jobSuccessful

        rrTableClient.setString(1, row, str(was_user_ever_logged_out))

        rrTableClient.setFloat(2, row, cpu_amount_OS)
        rrTableClient.setFloat(3, row, gpu_total)
        rrTableClient.setFloat(4, row, system_memory_total)

        rrTableClient.setInt(5, row, frame_count)
        rrTableClient.setInt(6, row, job_crash_count)
        rrTableClient.setInt(7, row, job_abort_count)
        rrTableClient.setInt(8, row, job_success_count)
        row += 1


print (statFolder + ": " + str(loaded) + " files loaded.")

out_csv = "{0}stats_{1}_{2}d.csv".format(exportFolder, datetime.now().strftime('%Y_%m_%d'), num_days)
if not rrTableClient.saveAsCsv(out_csv, False):
    print ("Error: Unable to save user table! " + rrTableClient.fileError())


print("Saved into folder " + exportFolder)
print("\n\n--- DONE ---\n\n")


