# -*- coding: cp1252 -*-
overrideIcon= ""

######################################################################
#
# Name:   
# Author:  Royal Render, Holger Schoenberger, Binary Alchemy
# Version: 
#
# Copyright (c) Holger Schoenberger - Binary Alchemy
######################################################################

#BUGS:    
#UI:
#Wait for file: no _wait folder, use variables
#
#console:
#Wait for file: replace variables


import os


class _TestFiles:
    def __init__(self, name=""):
        self.name = name
        self.description = ""
        self.hasInitFile = False
        
    def read_test(self, folder):
        if not self.name:
            return False

        test_definition_file = os.path.join(folder, self.name + ".bst")
        if not os.path.isfile(test_definition_file):
            return False

        self.hasInitFile = os.path.isfile(os.path.join(folder, self.name + "_init.bst"))
        
        file_obj = open(test_definition_file, "r")
        first_line = file_obj.readline()
        file_obj.close()

        pos_desc = first_line.find("ShortDescription:")
        if pos_desc < 0:
            return True

        pos_desc = first_line.find(":", pos_desc)
        pos_desc = pos_desc + 1
        self.description = first_line[pos_desc:]
        self.description = self.description.strip()

        return True
        

def get_config_folder():
    return os.path.join(rrGlobal.rrRootPath(), "plugins/submitter_scriptedjobs/baSpeedTest/")


def get_test_files():
    config_base_folder = os.path.normpath(get_config_folder())

    for filename in os.listdir(config_base_folder):
        full_path = os.path.join(config_base_folder, filename)
        if os.path.isdir(full_path):
            new_test = _TestFiles(filename)
            if new_test.read_test(full_path):
                yield new_test
        

def create_interface():
    speed_test_UI = rrGlobal.getGenericUI()
    speed_test_UI.addItem(rrGlobal.genUIType.label, "infoLabel", "")
    speed_test_UI.setText("infoLabel", '''How does it work?
    The application baHddSpeedTest executes various read/write/copy tests.
    It can create log files and excel tables with the results
    Including informations like the IP of the fileserver or the network cards Jumbo Frames setting 
    Create your own test files with the application RR/bin/[OS]/baHddSpeedTest_UI.
    
    This script creates jobs in RR to execute these tests. Each test contains 2 jobs:
        The 1st job creates the required files/folders.
        The 2nd job starts the baHddSpeedTest command on all assigned clients.
    
    Most tests are based on multiple files.
    Once one file was executed, clients create a file named [network folder]/[configName]/done_[host]
    Then they wait for the next config file in [network folder] to be executed
    
    Most tests are intended to run at the same time on all clients.
    To ensure this, the next config file is located in the folder [network folder]/_wait/
    Clients wait until you move the next config file from [network folder]/_wait/ into [network folder]/
    (Of course you can just copy all files from _wait/ directly into the [network folder], but then
    you do not test your fileserver as well. In this case ignore most of the steps in '2) Execution'  )
    
    How to use:
    1) Submission
      a) Choose a test and a [network folder] on this page
      b) Assign clients that should run the test
      c) Submit the jobs
      
      
    2) Execution
      a) Wait until the 1st 'init' job was executed on a client (if this job exists)
      b) Ensure all assigned clients will be able to execute the test job
         (Which means that you might need to deassign them from any active job and abort these jobs)
      c) Enable the 2nd job and let clients get the job
      d) Wait until all files [network folder]/_[configName]_log/done__[host] exist
      e) Copy the FIRST file from [network folder]/_wait/ into [network folder]/
      f) Repeat d) and e) until all config files have been executed\n'''
                          )

    speed_test_UI.addItem(rrGlobal.genUIType.spacerV, "spacerB", "")
    speed_test_UI.addItem(rrGlobal.genUIType.label, "labelChoose", "")
    speed_test_UI.setText("labelChoose", "\nPlease choose one of the speed tests:")
    speed_test_UI.addItem(rrGlobal.genUIType.dropdown, "testFile", "")

    for test_file in get_test_files():
        speed_test_UI.addText("testFile", test_file.name + " - " + test_file.description)

    speed_test_UI.addItem(rrGlobal.genUIType.spacerV, "spacerD", "")

    speed_test_UI.addItem(rrGlobal.genUIType.label, "labelDir", "")
    speed_test_UI.setText("labelDir", "Please specify the [network folder] for all config and test files:")
    speed_test_UI.addItem(rrGlobal.genUIType.browseDir, "netFolder", "")

    #Todo: Remove for final release and enable exe folder in next Todo
    speed_test_UI.addItem(rrGlobal.genUIType.label, "labelExe", "")
    speed_test_UI.setText("labelExe", "baHddSpeedTest executable:")
    speed_test_UI.addItem(rrGlobal.genUIType.browseFile, "exeFile", "")
    speed_test_UI.setText("exeFile", "<OsxApp \"<rrBin64>baHddSpeedTest\" >")

    speed_test_UI.addItem(rrGlobal.genUIType.spacerV, "spacerZ", "")
    speed_test_UI.addItem(rrGlobal.genUIType.layoutH, "btnLayout", "")
    speed_test_UI.addItem(rrGlobal.genUIType.closeButton, "Create Jobs", "btnLayout")
    speed_test_UI.addItem(rrGlobal.genUIType.closeButton, "Abort", "btnLayout")

    return speed_test_UI


def get_render_app(execute_type=rrGlobal.executeJobType.onceEachClient):
    render_app = rrJob._RenderAppBasic()
    render_app.clear()
    render_app.name = "Execute"
    render_app.type = "Execute"
    render_app.executeType = execute_type

    return render_app


def create_new_job(extension="", suffix="", execute_type=rrGlobal.executeJobType.onceEachClient, cfgFolder=""):
    newJob = rr.getNewJob()
    newJob.overrideIcon = overrideIcon
    newJob.sceneName = "<rrLocalBin><OsxApp baHddSpeedTest>"
    newJob.customSet_Str("CustomFlags", cfgFolder + selectedTest.name + extension)
    newJob.renderApp = get_render_app(execute_type=execute_type)
    newJob.companyProjectName = "rrCommand"
    newJob.imageFileName = os.path.join(cfgFolder, "no.file")
    newJob.imageSingleOutputFile = True
    newJob.splitImageFileInto_DirFileExt(False)
    newJob.uiIsChecked = True
    newJob.layer = selectedTest.name + suffix

    return newJob


def add_job(new_job):
    # If the index of the job is higher than the number of jobs,
    # then a new job is added to the end of the job list. So we use 999

    rr.jobAll_set(999, new_job)


if __name__ == "__main__":
    speed_test_UI = create_interface()
    speed_test_UI.execute()

    if speed_test_UI.value("Abort") == 1:
        del speed_test_UI
        raise rrCleanExit()

    netFolder = speed_test_UI.text("netFolder")
    selectedTestFile = speed_test_UI.text("testFile")
    selectedTestFile = selectedTestFile.split("-")[0]
    selectedTestFile = selectedTestFile.strip()

    exeFile = speed_test_UI.value("exeFile")
    del speed_test_UI

    if len(netFolder) == 0:
        rrGlobal.writeLog(rrGlobal.logLvL.warning, "No network folder specified!\nAborting.", "")
        raise rrCleanExit()

    netFolder = netFolder+"/"
    netFolder = netFolder.replace("\\","/")

    selectedTest = _TestFiles()
    for testFile in get_test_files():
        if testFile.name == selectedTestFile:
            selectedTest = testFile

    rrCopyExePath = rrGlobal.rrBinPath() + "rrCopy"
    os.system(rrCopyExePath + ' -d -oa ' + get_config_folder() + selectedTest.name + "    " + netFolder)

    # Create the jobs
    if selectedTest.hasInitFile:
        new_job = create_new_job(extension="_init.bst", suffix="Init", execute_type=rrGlobal.executeJobType.once, cfgFolder=netFolder)
        new_job.preID = 0
        new_job.customDataAppend_Str("rrSubmitterParameter", "\"CustomUserInfo=0~0~"
            "a) Wait until the first 'init' job was executed\n"
            "b) Ensure all assigned clients will be able to execute the second test job\n"
            "   (Which means that you might need to deassign them from any active job and abort these jobs)\n"
            "c) Enable the 2nd job and let clients get the job\""
            )
        add_job(new_job)

    new_job = create_new_job(extension=".bst", cfgFolder=netFolder)
    new_job.waitForPreID_set(0, 0)
    new_job.customDataAppend_Str("rrSubmitterParameter", "\"CustomUserInfo=0~0~"
      "d) Wait until all files "+netFolder+"_"+selectedTest.name+"_log/done__[host] exist\n"
      "e) Copy the FIRST file from "+netFolder+"_wait/ into "+netFolder+"\n"
      "f) Repeat d)'wait' and e)'copy' until all config files have been executed\n\""
            )
    new_job.customDataAppend_Str("rrSubmitterParameter", " SendJobDisabled=0~1")
    add_job(new_job)
